;***********************************************************
;*                 PRG2051.ASM  V1.0                       *
;* Programmer fuer AT89C2051, Anschluss an serielle        *
;* Schnittstelle eines PCs:                                *
;* 1200 Baud, 8 Datenbits, keine Paritaet, 1 Stoppbit      *
;* Assemblerprogramm fuer AT89S8252/AT89C51/52             *
;* Quarzfrequenz 11,0592 MHz                               *
;***********************************************************
;* World Of Electronics - Informationen rund um Elektronik *
;* http://www.woe.de.vu                                    *
;* woe@topmail.de                                          *
;* (c) 2000, WOE                                           *
;***********************************************************

XTAL1 equ p1.4  ; XTAL1
PROG  equ p1.3  ; /Prog
p_33  equ p1.2
p_34  equ p1.1
p_35  equ p1.0

dat   equ p0          ; Data an P0

taster equ p1.6       ; Taster
led equ p1.7          ; Status-LED

VCC_off equ p3.4      ; Steuerleitungen
VPP_off equ p3.5
VPP_5V equ p3.7

bcd1 equ p3.2         ; BCD-Schalter
bcd2 equ p3.3
bcd4 equ p1.5

rem_mode equ 0        ; Flag fuer Remote-Mode
rem_code equ 21h      ; Byte fuer Remote-Code


        org 0         ; Reset-Einsprungadresse
        jmp start

        org 0Bh       ; Timer 0 Interrupt
        jmp t0_int


start:
        clr rem_mode
        call init         ; Initialisierung

b1200:
	mov th1,#0e8h     ; 1200 Baud
	mov tl1,#0e8h
	mov tmod, #21h
	mov scon, #52h
	setb tr1
	mov a, sbuf
	clr ri

        setb ea           ; globale Interruptfreigabe
        setb et0          ; Timer 0 Interrupt freigeben

        call get_bcd      ; BCD-Schalter abfragen
        cjne a,#0,loop
        jmp remote_mode   ; BCD-Schalter auf 0 => Remote-Mode


loop:
	call wait_start   ; warten auf Tastendruck
        clr a
        jmp @a+dptr       ; in gewaehlten Programmteil verzweigen


program:                  ; Program-Mode
        setb led
        call chip_erase   ; Flash loeschen

        clr led
p_wait:
        jnb ri,p_wait     ; auf serielle Daten warten

	mov a,sbuf
	clr ri
        setb led          ; LED aus

        clr VCC_off       ; VCC on
        call delay_10ms
        clr VPP_off       ; RST=High
        setb PROG

        clr p_33          ; Program-Mode einstellen
        setb p_34
        setb p_35

        clr VPP_5V        ; VPP=12V
        call delay_1ms

prog_byte:
        cjne a,#0ffh,prog_1  ; Bytes mit FFh brauchen nicht
        jmp xtal_pulse       ;   programmiert zu werden
prog_1:
        mov dat,a
        clr PROG             ; Programmierimpuls
        nop
        setb PROG
        call wait_prog       ; Programmierzeit abwarten
xtal_pulse:
        setb XTAL1           ; internen Adresscounter erhoehen
        clr XTAL1
        call ser_wdh         ; auf weitere Daten warten
        jnc prog_byte
        jmp ser_end          ; nach Timeout Program-Mode beenden


verify:                      ; Verify-Mode
        clr led
v_wait:
        jnb ri,v_wait        ; auf serielle Daten warten
        mov a,sbuf
        clr ri
        setb led
        clr VCC_off
        call delay_10ms
        clr VPP_off
        setb PROG
        mov dat,#0ffh
        clr p_33             ; Verify-Mode einstellen
        clr p_34
        setb p_35
verify_byte:
        mov b,a
        mov a,dat
        cjne a,b,v_failed    ; auf Verify-Error pruefen
        setb XTAL1           ; internen Adresscounter erhoehen
        clr XTAL1
        call ser_wdh         ; auf weitere Daten warten
        jnc verify_byte
        jmp ser_end          ; nach Timeout Verify-Mode beenden

v_failed:                    ; Verify-Error aufgetreten
        setb VPP_5V   ; VPP=5V
        call delay_10ms
        setb VPP_off  ; VPP off
        clr p_33
        clr p_34
        clr p_35
        mov dat,#0
        clr PROG
        setb VCC_off  ; VCC off
        jnb rem_mode,v_wait_no_key  ; Remote-Mode?
        jmp rem_v_failed
v_wait_no_key:
        jnb taster,v_wait_no_key
v_wait_key:                    ; auf Tastendruck warten
        jb taster,v_wait_key
        nop
        nop
        jb taster,v_wait_key
        jmp loop               ; Verify-Mode beenden


read:                          ; Read-Mode
        setb led
        clr VCC_off
        call delay_10ms
        clr VPP_off
        setb PROG
        mov dat,0ffh
        clr p_33               ; Read-Mode einstellen
        clr p_34
        setb p_35
        mov r6,#8              ; 2048 Bytes auslesen (8*256)
read_byte:
        mov r5,#0
read256:
        mov a,dat
        call sendbyte          ; gelesenes Byte seriell versenden
        setb XTAL1             ; internen Adresscounter erhoehen
        clr XTAL1
        djnz r5,read256
        djnz r6,read_byte
        jmp ser_end            ; Read-Mode beenden


lockbits:                   ; Lockbits-Mode
        clr led
l_wait:
        jnb ri,l_wait       ; auf serielles Datum warten
        mov a,sbuf
        clr ri
        cjne a,#49,lb_1     ; '1' = Lockbit 1 programmieren
        jmp lock
lb_1:
        cjne a,#50,l_wait   ; '2' = Lockbits 1 & 2 progr.
lock:
        setb led
        clr VCC_off         ; VCC on
        call delay_10ms
        clr VPP_off         ; RST=High
        setb PROG

        setb p_33           ; Lockbit 1
        setb p_34
        setb p_35

        clr VPP_5V          ; VPP=12V
        call delay_1ms
        clr PROG            ; Programmierimpuls
        nop
        setb PROG
        call wait_prog      ; Programmierzeit abwarten

        cjne a,#50,lock_end ; Lockbit 2 programmieren?
        clr p_35            ; Lockbit 2
        clr PROG            ; Programmierimpuls
        nop
        setb PROG
        call wait_prog      ; Programmierzeit abwarten
lock_end:
        jmp ser_end         ; Lockbits-Mode beenden
        


ser_end:                        ; Ende eines eingestellten Modes
        setb VPP_5V   ; VPP=5V  ; Controller spannungslos schalten
        call delay_10ms
        setb VPP_off  ; VPP off
        clr p_33
        clr p_34
        clr p_35
        mov dat,#0
        clr PROG
        setb VCC_off  ; VCC off
        jnb rem_mode,no_remote  ; Remote-Mode?
        jmp remote_ret
no_remote:
        jmp loop



ser_wdh:                ; auf serielle Daten warten
        mov r4,#8       ; Timeout nach ca. 2s (Carry-Flag=1)
w3:
        mov r3,#0
w2:
        mov r2,#0
w1:
	jb ri,ri_ok
        djnz r2,w1
        djnz r3,w2
        djnz r4,w3
        setb c          ; Timeout ueberschritten
        jmp s_end
ri_ok:                  ; Zeichen empfangen
	mov a,sbuf
	clr ri
        clr c
s_end:
        ret



wait_prog:              ; max. Programmierzeit von 2ms warten
        call delay_1ms
        call delay_1ms
	ret


remote_mode:                 ; Remote-Mode
        setb rem_mode
        mov th0,#0
        mov tl0,#0
        mov r7,#5
        cpl led
        setb tr0             ; LED blinkt per Interrupt
rem_wait:
        jnb ri,rem_wait      ; warten auf serielle Daten
        mov a,sbuf
        clr ri
        cjne a,#0D0h,rem_wait  ; Check: D0h empfangen?
        clr tr0
        setb led             ; LED aus
rem_wait2:
        jnb ri,rem_wait2     ; warten auf Remote-Code
        mov a,sbuf
        clr ri
        cjne a,#0A0h,rem_l1  ; Code A0h: Chip Erase
        mov rem_code,#50h
        call chip_erase
        jmp remote_ret
rem_l1:        
        cjne a,#0A1h,rem_l2  ; Code A1h: Program
        mov rem_code,#51h
        jmp p_wait
rem_l2:
        cjne a,#0A2h,rem_l3  ; Code A2h: Verify
        mov rem_code,#52h
        jmp v_wait
rem_l3:
        cjne a,#0A3h,rem_l4  ; Code A3h: Read
        mov rem_code,#53h
        jmp read
rem_l4:
        cjne a,#0A4h,rem_l5  ; Code A4: Lockbits
        mov rem_code,#54h
        jmp l_wait
rem_l5:
        cjne a,#0AAh,rem_l6  ; Code AAh: Communication check
        mov rem_code,#5Ah
        jmp remote_ret
rem_l6:
        jmp rem_wait2

remote_ret:                  ; Rueckmelde-Code senden
        mov a,rem_code
        cjne a,#53h,rem_send ; bei Read keine Rueckmeldecode!
        jmp remote_mode
rem_send:
        call sendbyte
        jmp remote_mode      ; auf neuen Remote-Code warten


rem_v_failed:                ; Verify-Error im Remote-Mode
        mov a,#5Fh           ; Rueckmeldecode 5Fh senden
        call sendbyte
wait_timeout:
        call ser_wdh         ; warten bis Timeout bei seriellem
        jnc wait_timeout     ; Datenempfang
        mov a,#5Eh           ; dann Rueckmeldecode 5Eh senden
        call sendbyte
        clr led              ; LED leuchtet anstatt zu blinken
        jmp rem_wait


init:                ; Initialisierung
        setb VPP_5V  ; VPP=5V
        setb VPP_off ; VPP off
        setb VCC_off ; VCC off
        clr XTAL1    ; XTAL1=Low
        clr PROG     ; PROG=Low
        clr p_33     ; prog. mode bits=Low
        clr p_34
        clr p_35
        mov dat,#0
        ret


chip_erase:               ; Flash loeschen
        clr VCC_off       ; VCC on
        call delay_10ms
        clr VPP_off       ; RST=High
        setb PROG

        setb p_33         ; Chip Erase Mode einstellen
        clr p_34
        clr p_35
        clr VPP_OFF       ; VPP on
        call delay_10ms
        clr VPP_5V        ; VPP=12V
        call delay_10ms
        clr PROG          ; Programmierimpuls 10ms
        call delay_10ms
        setb PROG
        setb VPP_5V       ; VPP=5V
        call delay_10ms
        setb VPP_off      ; VPP off
        clr p_33
        clr p_34
        clr p_35
        mov dat,#0
        clr PROG
        setb VCC_off      ; VCC off
        ret


delay_1ms:         ; mind. 1ms warten (fosc=11.0592 MHz)
        push 1
        mov r1,#235
d1_1:
        nop
        nop
        djnz r1,d1_1
        pop 1
        ret


delay_10ms:         ; mind. 10ms warten (fosc=11.0592 MHz)
        push 1
        push 2
        mov r1,#44
d10_1:
        mov r2,#100
d10_2:
        djnz r2,d10_2
        djnz r1,d10_1
        pop 2
        pop 1
        ret


wait_start:       ; warten bis eingestellter Mode gestartet wird
        mov th0,#0
        mov tl0,#0
        mov r7,#5
        cpl led
        setb tr0     ; LED blinkt per Timer-Interrupt

wait_no_key:
        jnb taster,wait_no_key  ; warten bis letzter Tastendruck beeendet
wait_key:
        jb taster,wait_key      ; warten auf Tastendruck
        nop
        nop
        jb taster,wait_key
key:
        call get_bcd        ; BCD-Schalter abfragen
        cjne a,#1,l_1
        mov dptr,#program   ; 1 = Program
        jmp l_end
l_1:
        cjne a,#2,l_2
        mov dptr,#verify    ; 2 = Verify
        jmp l_end
l_2:
        cjne a,#3,l_3
        mov dptr,#read      ; 3 = Read
        jmp l_end
l_3:
        cjne a,#4,l_4
        mov dptr,#lockbits  ; 4 = Lockbits
        jmp l_end
l_4:
        call delay_1ms
        jmp wait_no_key     ; restliche Stellungen nicht belegt

l_end:
        clr tr0             ; LED-Blinken stoppen
        clr ri
        ret


get_bcd:                    ; BCD-Schalter abfragen
       clr a                ; (Rueckgabewert im Akku)
       jb bcd1,b1
       inc a
b1:
       jb bcd2,b2
       add a,#2
b2:
       jb bcd4,b3
       add a,#4
b3:
       ret


sendbyte:                   ; serielles Datum versenden (Akku)
s_wait:
       jnb ti,s_wait
       clr ti
       mov sbuf,a
       ret


t0_int:                     ; Timer0-Interruptroutine (LED-Blinken)
      djnz r7,t0_end
      cpl led
      mov r7,#5
t0_end:      
      reti


END
